/* minimal program to open & clear a window */
#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"
#include "geometryPositionColor.h"

using namespace moglclass;

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

/*Triangle *tri1;
Triangle *tri2;
Sphere *sph1;
Cube *cube1;
Cylinder *cyl1;
Torus *tor1;*/

int startTime = 0;
std::vector<GeometryPositionColor*> geoList;
Sa * sa1;

bool g_mousemove = false;
bool g_wiremode = false;
bool g_animationmode = false;
int g_geometrymode = 0;

float g_fovy = 45.0f;
float g_aspect = 1.0f;
float g_zNear = 1.0f;
float g_zFar = 100.f;

glm::vec3 g_eye(0, 0, 10.0f);
glm::vec3 g_at(0, 0, 0);
glm::vec3 g_up(0, 1, 0);

glm::mat4 World(1.0f);
glm::mat4 View(1.0f);
glm::mat4 Projection(1.0f);
glm::mat4 MVP(1.0f);

glm::mat4 yRotation(1.0f);

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

// set geometry
void setData()
{
	geoList.push_back(new Quad());
	geoList.push_back(new Triangle());
	geoList.push_back(new Circle());
	geoList.push_back(new Sphere(glm::vec3(0.0f, 0.5f, 0.0f), 0.5f, 16, 16));
	geoList.push_back(new Cube(glm::vec3(0.0f, -0.7f, 0.0f), 0.5f));
	geoList.push_back(new Cylinder(glm::vec3(0.0f, -0.3f, 0.0f), 0.5, 0.5, 16));
	geoList.push_back(new Torus(glm::vec3(0.0f, -1.0f, 0.0f), 1.5f, 0.5f, 16, 16));
	geoList.push_back(new Parallelepiped(glm::vec3(-1.0f, -1.0f, 0.0f), glm::vec3(0.5f, 0.0f, 0.0f), glm::vec3(0.0f, 0.5f, 0.0f), glm::vec3(1.0f, 2.0f, 0.0f)));
	sa1 = new Sa();
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple3.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple3.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();

	setData();

	printf("setAnimation\n");
	//geoList[0]->setAnimation(glm::vec3(-1, 0, 0), glm::vec3(1, 0, 0), 2000);
	//geoList[1]->setAnimation(glm::vec3(-2, -2, 0), glm::vec3(2, 2, 0), 2000);
	//geoList[2]->setAnimation(glm::vec3(2, -2, 0), glm::vec3(-2, 2, 0), 2000);
	
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT ); // clear the window

		// set drawmode
	if (g_wiremode) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// MVP matrix
	Projection = glm::perspective(g_fovy, g_aspect, g_zNear, g_zFar);
	View = glm::lookAt(g_eye, g_at, g_up);

	World = yRotation;
	MVP = Projection * View * World;
	spMain.useProgram();
	spMain.setUniform("gMVP", MVP);

	geoList[g_geometrymode]->draw();
	sa1->draw();
	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

	glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;
	elapsedTime = currentTime - startTime;

	if (g_animationmode)
	{
		// translate & rotate around x/y axis
		static float angle = 0.0f;
		angle += 0.001f * deltaTime;

		// geometry translate & rotate
		glm::mat4 xRotation;
		xRotation = glm::rotate(glm::mat4(1.0f), angle, glm::vec3(1, 0, 0));
		yRotation = glm::rotate(xRotation, angle, glm::vec3(0, 1, 0));
	}

	// each geometry update
	geoList[g_geometrymode]->update((float)elapsedTime);
	if (geoList[g_geometrymode]->isAnimationDone())
	{
		g_geometrymode++;
		if (g_geometrymode == 8) 
		{
			g_geometrymode = 0;
			for (int i=0; i<(int)geoList.size(); i++)	geoList[i]->reset(); // δ ׷ 
		}

		geoList[g_geometrymode]->activate(); // ִϸ̼ ϳ     ִϸ̼ activate
		startTime = glutGet(GLUT_ELAPSED_TIME);
		printf("geoList[%d] animation is activate\n", g_geometrymode);
	}
	sa1->update((float)elapsedTime);

	elapsedTime = currentTime - startTime;
	glutPostRedisplay();
}

void keyboard( unsigned char key, int x, int y )
{
	switch ( key ) {
		case 27:
			exit( EXIT_SUCCESS );
			break;
		case 'c':
			geoList[g_geometrymode]->setColor(glm::vec3(1, 0, 0));
			break;
		case 'w':
			g_wiremode = !g_wiremode;
			break;
		case 'a':
			g_animationmode = !g_animationmode;
			break;
		case 'g':
			g_geometrymode = (g_geometrymode + 1) % 8;
			printf("g_geometrymode=%d\n", g_geometrymode);
			break;
		case 'm':
			startTime = glutGet(GLUT_ELAPSED_TIME);
			geoList[g_geometrymode]->activate();
			break;
		case 'n':
			startTime = glutGet(GLUT_ELAPSED_TIME);
			sa1->activate();
			break;
	}
	glutPostRedisplay();
}

void mouse(int button, int state, int mx, int my)
{
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN)
		exit(0);
	if (button==GLUT_LEFT_BUTTON && state==GLUT_DOWN)
		g_mousemove = true;
	else if (button==GLUT_LEFT_BUTTON && state==GLUT_UP)
		g_mousemove = false;
}

void motion(int mx, int my)
{
	int w = glutGet(GLUT_WINDOW_WIDTH);
	int h = glutGet(GLUT_WINDOW_HEIGHT);

	// 0~600(x+ right) x 0~600(y+ down) => -5~5(x+ right) x -5~5(y+ up)
	float x = (float)  10.0f * (mx - w*0.5f) / w;
	float y = (float) 10.0f * (h*0.5f - my) / h;

	if (g_mousemove)
	{
		//printf("mx=%d my=%d x=%f y=%f\n", mx, my, x, y);
		//geoList[g_geometrymode]->setPosition(glm::vec3(x, y, 0));
		sa1->setPosition(glm::vec3(x, y, 0));
	}

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	g_aspect = (float) (w/h);

    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glutPostRedisplay();
}

void cleanup()
{		 
	spMain.deleteProgram();
}

int main( int argc, char **argv )
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA );
	glutInitWindowSize( 600, 600 );
	glutCreateWindow( "Geometry Primitives" );

	glewInit();
	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutMouseFunc( mouse ); 
	glutMotionFunc( motion ); 
	glutIdleFunc( update );
	glutReshapeFunc( reshape );
	glutMainLoop();

	cleanup();
	return 0;
}